from Components.ActionMap import NumberActionMap, ActionMap
from Components.Pixmap import Pixmap
from Components.FileList import FileList
from Components.MenuList import MenuList
from Components.ConfigList import ConfigList
from Components.config import *
from Components.Label import Label
from Screens.Screen import Screen
from Components.AVSwitch import AVSwitch
from Screens.MessageBox import MessageBox
from Screens.Console import Console
from Plugins.Plugin import PluginDescriptor
from Plugins.Extensions.PicturePlayer.plugin import PicView
from Tools.Directories import resolveFilename, fileExists, pathExists, createDir, SCOPE_MEDIA
from Tools import Notifications
from ServiceReference import ServiceReference
from Screens.InputBox import InputBox
from Screens.ChoiceBox import ChoiceBox
from Components.Input import Input
from Components.Button import Button
from enigma import eTimer, eWidget, eRect, loadPic, eServiceReference
from Components.ConfigList import ConfigListScreen
import os

config.efb = ConfigSubsection()
config.efb.lastDirR = ConfigText(default=resolveFilename(SCOPE_MEDIA))
config.efb.lastDirG = ConfigText(default=resolveFilename(SCOPE_MEDIA))
config.efb.lastDirB = ConfigText(default=resolveFilename(SCOPE_MEDIA))
config.efb.lastDirY = ConfigText(default=resolveFilename(SCOPE_MEDIA))
config.efb.lastMode = ConfigInteger(default=0, limits=(0, 3))
config.efb.mvi_delay = ConfigInteger(default=3, limits=(3, 100))
config.efb.showMvi = ConfigYesNo(default=True)

def getAspect():
	val = AVSwitch().getAspectRatioSetting()
	return val/2

class FileOptions(Screen):
	skin = """
		<screen position="230,115" size="250,200" title="File Options" >
			<ePixmap name="key1" position="10,10" zPosition="4" size="30,30" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/MerlinBrowser/icons/key_1.png" transparent="1" alphatest="on" />
			<ePixmap name="key2" position="10,40" zPosition="4" size="30,30" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/MerlinBrowser/icons/key_2.png" transparent="1" alphatest="on" />
			<ePixmap name="key3" position="10,70" zPosition="4" size="30,30" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/MerlinBrowser/icons/key_3.png" transparent="1" alphatest="on" />
			<ePixmap name="key4" position="10,100" zPosition="4" size="30,30" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/MerlinBrowser/icons/key_4.png" transparent="1" alphatest="on" />
			<ePixmap name="key5" position="10,130" zPosition="4" size="30,30" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/MerlinBrowser/icons/key_5.png" transparent="1" alphatest="on" />
			<widget name="chmod" position="40,10" size="170,25" halign="left" font="Regular;20" zPosition="5" transparent="1" />
			<widget name="copy"  position="40,40" size="170,25" halign="left" font="Regular;20" zPosition="5" transparent="1" />
			<widget name="paste" position="40,70" size="170,25" halign="left" font="Regular;20" zPosition="5" transparent="1" />
			<widget name="rem" position="40,100" size="170,25" halign="left" font="Regular;20" zPosition="5" transparent="1" />
			<widget name="ren" position="40,130" size="170,25" halign="left" font="Regular;20" zPosition="5" transparent="1" />
		</screen>"""

	def __init__(self, session, filename, currentDir, descent, clid, args = None):
		self.skin = FileOptions.skin
		Screen.__init__(self, session)
		
		
		self.clipData = clid[0]
		self.clipDataDir = clid[1]
		self.dataInClipboard = None
		
		if not self.clipData is None:
			self.dataInClipboard = True

		self.filename = filename
		self.currentDir = currentDir
		self.descent = descent
		
		if self.descent and not self.dataInClipboard:
			self["chmod"] = Button(_("Chmod disabled"))
			self["copy"] = Button(_("Copy disabled"))
			self["paste"] = Button(_("Paste disabled"))
			self["rem"] = Button(_("Delete disabled"))
			self["ren"] = Button(_("Rename disabled"))
		elif self.descent and self.dataInClipboard:
			self["chmod"] = Button(_("Chmod disabled"))
			self["copy"] = Button(_("Copy disabled"))
			self["paste"] = Button(_("Paste"))
			self["rem"] = Button(_("Delete disabled"))
			self["ren"] = Button(_("Rename disabled"))
		elif not self.descent and self.dataInClipboard:
			self["chmod"] = Button(_("Chmod"))
			self["copy"] = Button(_("Copy"))
			self["paste"] = Button(_("Paste"))
			self["rem"] = Button(_("Delete"))
			self["ren"] = Button(_("Rename"))
		else:
			self["chmod"] = Button(_("Chmod"))
			self["copy"] = Button(_("Copy"))
			self["paste"] = Button(_("Paste disabled"))
			self["rem"] = Button(_("Delete"))
			self["ren"] = Button(_("Rename"))

		self["myactions"] = ActionMap(["ColorActions", "OkCancelActions", "MenuActions", "InfobarActions"],
		{
			"ok": self.Exit,
			"cancel": self.Exit
		}, -1)
		
		self["actions"] = NumberActionMap(["SetupActions"],
			{
				"1": self.keyGlob,
				"2": self.keyGlob,
				"3": self.keyGlob,
				"4": self.keyGlob,
				"5": self.keyGlob

			}, -1)
		
	def keyGlob(self, key):
		if key == 1:
			if not self.descent:
				self.session.openWithCallback(self.chmod, InputBox, title=_("Enter Number"), windowTitle =_("CHMOD"), maxSize=False, visible_width = 56, type=Input.NUMBER)
		elif key == 2:
			if not self.descent:
				self.copy()
		elif key == 3:
			self.paste()
		elif key == 4:
			if not self.descent:
				tlist = []
				tlist.append((_("Delete Selection"), "delete"))
				tlist.append((_("Discard"), "abort"))
				self.session.openWithCallback(self.delete, ChoiceBox, title=_("Really delete %s " % self.filename), list = tlist)
		elif key == 5:
			if not self.descent:
				self.rename()
			
	def delete(self, action):
		if action[1] == 'delete':
			order = 'rm -f ' + self.currentDir + self.filename
			print order
			try:
				os.system(order)
				self.success()
			except:
				info = self.session.open(MessageBox,_("File operation %s failed" % order), MessageBox.TYPE_ERROR)
				info.setTitle(_("Delete"))

	def copy(self):
		self.clipData = self.filename
		self.clipDataDir = self.currentDir
		print self.clipDataDir + self.clipData
		self.dataInClipboard = True
		self["paste"].setText("Paste")
		self.success()

	def paste(self):
		if self.dataInClipboard:
			source = self.clipDataDir + self.clipData
			dest = self.currentDir + self.clipData
			for x in dest:
				if x == ' ':
					self.session.open(MessageBox, _("Your Destinationpath contains whitespaces.\nNo Paste possible!"), MessageBox.TYPE_ERROR)
					return
				try:
					order = 'cp ' + source + " " + dest
					os.system(order)
					self.success()
				except:
					info = self.session.open(MessageBox,_("Selection: %s\nFile operation failed" % source), MessageBox.TYPE_ERROR)
					info.setTitle(_("Paste"))
		else:
			info = self.session.open(MessageBox,_("Sorry, there is nothing to paste..."), MessageBox.TYPE_ERROR)
			info.setTitle(_("Paste"))
	
	def chmod(self, number):
		if number[0] != 0:
			newnumber = '0' + number
		else:
			newnumber = number
		try:	
			what = self.currentDir + self.filename
			os.system('chmod ' + newnumber + ' ' + what)
			self.success()
		except:
			info = self.session.open(MessageBox,_("Selection: %s\nFile operation failed" % self.filename), MessageBox.TYPE_ERROR)
			info.setTitle(_("Chmod"))
	
	def rename(self):
		self.showRenameInputBox(self.filename)

	def showRenameInputBox(self, filename):
		if not filename is None:
			self.session.openWithCallback(self.renameInputCallback, InputBox, title=_("Please enter a new filename"), windowTitle =_("Rename"), text=filename, maxSize=False, visible_width = 56, type=Input.TEXT)
	
	def renameInputCallback(self, text):
		if not text is None:
			source = self.currentDir + self.filename
			dest = self.currentDir + text
			try:
				os.rename(source, dest)
				self.success()
			except:
				info = self.session.open(MessageBox,_("Selection: %s\nFile operation failed" % self.filename), MessageBox.TYPE_ERROR)
				info.setTitle(_("Rename"))
	
	def success(self):
		info = self.session.open(MessageBox,_("Selected operation where successfull!"), MessageBox.TYPE_INFO)
		info.setTitle(_("Information"))
	
	def Exit(self):
		retval = []
		retval.append(self.clipData)
		retval.append(self.clipDataDir)
		self.close(retval)
	
class FileEdit(Screen):
	skin = """
		<screen position="60,75" size="600,450" title="File Editor" >
			<widget name="zeilen" position="10,10" size="580,430" scrollbarMode="showOnDemand" />
			<widget name="pixmap" position="200,0" size="190,250" alphatest="on" />
		</screen>"""
		
	def __init__(self, session, file):
		self.skin = FileEdit.skin
		Screen.__init__(self, session)
		
		self.eFile = file
		self["pixmap"] = Pixmap()
#		self["red"] = Label()

		self["myactions"] = ActionMap(["OkCancelActions"],
		{
			"ok": self.edit,
			"cancel": self.Exit
		}, -1)
		
		self.list = []
		self["zeilen"] = MenuList(self.list)
		self.getFile(file)
		self.lastLine = None
		self.lastSel = None
		self.isChanged = None
			
	def getFile(self, file):
		liste = []
		try:
			fliste = open(file, "r")
		except:
			return None
		
		if not fliste is None:
			for line in fliste:
				self.list.append(line)
			fliste.close()

	def edit(self):
		self.lastSel = self["zeilen"].getSelectionIndex()
		self.showEditInputBox(self.list[self.lastSel])

	def showEditInputBox(self, line):
		if not line is None:
			self.lastLine = line
			self.session.openWithCallback(self.editInputCallback, InputBox, title=_("Please enter a new filename"), windowTitle =_("Line Edit"), text=line, maxSize=False, visible_width = 56, type=Input.TEXT)
	
	def editInputCallback(self, text):
		if not text is None:
			for x in self.list:
				if x == self.lastLine:
					self.isChanged = True
					self.list.remove(x)
					self.list.insert(self.lastSel, text)
					
			
		self.lastLine = None
		self.lastSel = None
	
	def save(self, sel):
		if not sel is None:
			if sel[1] == 'save':
				self.isChanged = None
				if not self.eFile is None:
					try:
						eFile = open(self.eFile, "w")
						for x in self.list:
							eFile.writelines(x)
						eFile.close()
					except:
						pass
			else:
				self.isChanged = None
		
	def Exit(self):
		if not self.isChanged is None:
			tlist = []
			tlist.append((_("Save all Changes you made"), "save"))
			tlist.append((_("Discard Changes"), "abort"))
			self.session.openWithCallback(self.save, ChoiceBox, title=_("Should save changes?"), list = tlist)
		self.close(0)

class MB_Config(Screen, ConfigListScreen):
	skin = """
		<screen name="MBSet" position="80,140" size="560,330" title="Merlin Browser Settings">
			<widget name="config" position="10,10" size="540,250" scrollbarMode="showOnDemand" />
			<ePixmap name="red"    position="0,280"   zPosition="4" size="140,40" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/MerlinBrowser/icons/key_red.png" transparent="1" alphatest="on" />
			<ePixmap name="green"  position="140,280" zPosition="4" size="140,40" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/MerlinBrowser/icons/key_green.png" transparent="1" alphatest="on" />
			<ePixmap name="yellow" position="280,280" zPosition="4" size="140,40" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/MerlinBrowser/icons/key_yellow.png" transparent="1" alphatest="on" />
			<ePixmap name="blue"   position="420,280" zPosition="4" size="140,40" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/MerlinBrowser/icons/key_blue.png" transparent="1" alphatest="on" />
			<widget name="key_red" position="0,280" zPosition="5" size="140,40" valign="center" halign="center" font="Regular;21" transparent="1" foregroundColor="white" shadowColor="black" shadowOffset="-1,-1" />
			<widget name="key_green" position="140,280" zPosition="5" size="140,40" valign="center" halign="center" font="Regular;21" transparent="1" foregroundColor="white" shadowColor="black" shadowOffset="-1,-1" />
			<widget name="key_yellow" position="280,280" zPosition="5" size="140,40" valign="center" halign="center"  font="Regular;21" transparent="1" foregroundColor="white" shadowColor="black" shadowOffset="-1,-1" />
			<widget name="key_blue" position="420,280" zPosition="5" size="140,40" valign="center" halign="center" font="Regular;21" transparent="1" foregroundColor="white" shadowColor="black" shadowOffset="-1,-1" />
		</screen>"""
	
	def __init__(self, session):
		Screen.__init__(self, session)
		self["actions"] = ActionMap(["SetupActions", "ColorActions"],
		{
			"green": self.save,
			"red": self.Exit,
			"cancel": self.Exit
		}, -2)

		self["key_red"] = Button(_("Cancel"))
		self["key_green"] = Button(_("OK"))
		self["key_yellow"] = Button("")
		self["key_blue"] = Button("")

		cfglist = []
		cfglist.append(getConfigListEntry(_("Show MVI"), config.efb.showMvi))
		cfglist.append(getConfigListEntry(_("MVI delay"), config.efb.mvi_delay))
		ConfigListScreen.__init__(self, cfglist, session)

	def save(self):
		ConfigListScreen.keySave(self)
		configfile.save()
		
	def Exit(self):
		self.close(0)

class MerlinBrowser(Screen):
	skin = """
		<screen position="0,0" size="720,576" title="Merlin Browser" flags="wfNoBorder" backgroundColor="#1f1f21" >
			<ePixmap name="mb_bg" position="0,0" zPosition="-1" size="720,576" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/MerlinBrowser/mb_bg.png" />
			<widget name="red" halign="left" zPosition="-2" valign="center" position="80,50" size="130,20" font="Regular;20" transparent="1" foregroundColor="#f0f0f0" />
			<widget name="blue" halign="left" zPosition="-2" valign="center" position="531,50" size="130,20" font="Regular;20" transparent="1" foregroundColor="#f0f0f0" />
			<widget name="green" halign="left" zPosition="-2" valign="center" position="227,50" size="130,20" font="Regular;20" transparent="1" foregroundColor="#f0f0f0" />
			<widget name="yellow" halign="left" zPosition="-2" valign="center" position="379,50" size="130,20" font="Regular;20" transparent="1" foregroundColor="#f0f0f0" />
			<widget name="description" zPosition="5" transparent="1" halign="center" position="60,291" size="600,20" font="Regular;18" foregroundColor="#f0f0f0"/>
			<widget name="list" position="125,331" size="544,190" zPosition="5" transparent="1" foregroundColor="#f0f0f0" scrollbarMode="showOnDemand" />
			<widget name="root_list" position="305,50" size="364,234" zPosition="5" transparent="1" foregroundColor="#f0f0f0" scrollbarMode="showOnDemand" />
			<widget name="m_back" position="49,326" size="70,200" zPosition="4" alphatest="on" />
			<widget source="session.VideoPicture" render="Pig" position="49,84"  zPosition="5" size="250,200" backgroundColor="transparent" />
			<widget name="pixmap" position="10,10" size="120,60" alphatest="on" />
		</screen>"""
		
	def __init__(self, session, args = None):
		self.skin = MerlinBrowser.skin
		Screen.__init__(self, session)
		self.isVisible = True
		self.mode = int(config.efb.lastMode.value)
		
		self.filePattern = "^.*\.(mpeg|ts|mpg|Mpeg|MPEG|Ts|TS|Mpg|MPG)"
		self.musicPattern = "^.*\.(mp3|Mp3|MP3|ogg|Ogg|OGG|wav|Wav|WAV)"
		self.picturePattern = "^.*\.(png|Png|PNG|jpg|Jpg|JPG|jpe|Jpe|JPE|bmp|Bmp|BMP|jpeg|Jpeg|JPEG)"
		self.packagePattern = "^.*\.*"
		self.index = 0
		self.clipData = None
		self.clipDataDir = None
		self.onExit = False
		self.mp3Timer = eTimer()
		self.startTimer = eTimer()
		self.mviList = []
		self.mviIndex = 0
		self["root_list"] = []
		self["list"] = []
		self["pixmap"] = Pixmap()
		self["m_back"] = Pixmap()
		self["red"] = Button(_("Pictures"))
		self["green"] = Button(_("Music"))
		self["yellow"] = Button(_("Files"))
		self["blue"] = Button(_("Movies"))
		self["description"] = Label(_("Starting up.."))
		self.aspect = getAspect()
		
		curDir = None
		pattern = None
		
		if self.mode == 0:
			curDir = config.efb.lastDirB.value
			pattern = self.filePattern
			self["description"].setText(_("Movies-> " + curDir))
			
		elif self.mode == 1:
			curDir = config.efb.lastDirY.value
			pattern = self.packagePattern
			self["description"].setText(_("Files-> " + curDir))
			
		elif self.mode == 2:
			curDir = config.efb.lastDirR.value
			pattern = self.picturePattern
			self["description"].setText(_("Pictures-> " + curDir))
			
		elif self.mode== 3:
			curDir = config.efb.lastDirG.value
			pattern = self.musicPattern
			self["description"].setText(_("Music-> " + curDir))
			
		
		if not curDir is None:
			try:
				if self.mode != 1 and self.mode != 2:
					self["list"] = FileList(curDir, showDirectories = False, matchingPattern = pattern, useServiceRef = True)
				else:
					self["list"] = FileList(curDir, showDirectories = False, matchingPattern = pattern, useServiceRef = False)
			except:
				if self.mode != 1 and self.mode != 2:
					self["list"] = FileList("/", showDirectories = False, matchingPattern = pattern, useServiceRef = True)
				else:
					self["list"] = FileList("/", showDirectories = False, matchingPattern = pattern, useServiceRef = False)
				
		else:
			if self.mode != 1 and self.mode != 2:
				self["list"] = FileList("/", showDirectories = False, matchingPattern = self.filePattern, useServiceRef = True)
			else:
				self["list"] = FileList("/", showDirectories = False, matchingPattern = self.filePattern, useServiceRef = False)
		
		if not curDir is None:
			try:
				self["root_list"] = FileList(curDir, showDirectories = True, showFiles = False)
			except:
				self["root_list"] = FileList("/", showDirectories = True, showFiles = False)
		else:
			self["root_list"] = FileList("/", showDirectories = True, showFiles = False)
		
		self["root_list"].onSelectionChanged.append(self.updateChild)

		self["myactions"] = ActionMap(["ChannelSelectBaseActions", "ColorActions", "OkCancelActions", "MenuActions", "InfobarActions", "DirectionActions"],
		{
			"ok": self.ok,
			"cancel": self.Exit,
			"red": self.switchModeR,
			"green": self.switchModeG,
			"yellow": self.switchModeY,
			"blue": self.switchModeB,
			"menu": self.options,
			"up": self.up,
			"down": self.down,
			"nextBouquet": self.focusOnParent,
			"prevBouquet": self.focusOnChild,
			"left": self.pageUp,
			"right": self.pageDown,
			"showMovies": self.visibility
		}, -1)
		
		self.mp3Timer.callback.append(self.showBackgroundMVI)
		self.startTimer.callback.append(self.setStartMode)
		self.startTimer.start(True, 50)
		

	def setStartMode(self):
		self["m_back"].instance.setPixmapFromFile("/usr/lib/enigma2/python/Plugins/Extensions/MerlinBrowser/mode" + str(self.mode) + ".png")
		self.focusOnParent()
		
	def focusOnChild(self):
		self.currentList = "list"
		self["root_list"].selectionEnabled(0)
		self["list"].selectionEnabled(1)
	
	def focusOnParent(self):
		self.currentList = "root_list"
		self["list"].selectionEnabled(0)
		self["root_list"].selectionEnabled(1)
	
	def up(self):
		self[self.currentList].up()

	def down(self):
		self[self.currentList].down()
	
	def pageUp(self):
		self[self.currentList].pageUp()

	def pageDown(self):
		self[self.currentList].pageDown()
		
	def getMVI(self):
		path = "/usr/saver/"
		for root, dirs, files in os.walk(path):
			for name in files:
				if name.endswith(".mvi"):
					self.mviList.append(name)
		print self.mviList

	def showBackgroundMVI(self):
		if len(self.mviList) > 0:
			order = "/usr/bin/showiframe /usr/saver/" + self.mviList[self.mviIndex] + " &"
			if self.mviIndex < len(self.mviList) -1:
				self.mviIndex += 1
			else:
				self.mviIndex = 0
			print self.mviIndex
			print order
			os.system(order)
			time = config.efb.mvi_delay.getValue() * 1000
			self.mp3Timer.start(time, True)
			
	def ok(self):
		if self.isVisible == False:
			self.visibility()
			return
		self.mp3Timer.stop()
		
		if self.currentList == "root_list":
			if self[self.currentList].canDescent():
				try:
					self.updateChild(self[self.currentList].getCurrentDirectory())
					self[self.currentList].descent()
				except:
					return
			else:
				return
		if self.currentList == "list":
			self["pixmap"].instance.setPixmapFromFile(self["list"].getFilename())
			self.play()

	def updateChild(self, directory = None):
		if self.onExit == True:
			return
		print 'updating Child with directory %s ' % directory
		if directory is None:
			if self["root_list"].canDescent():
				directory = self["root_list"].getFilename()
				print 'cached directory %s ' % directory
			else:
				print 'preview of files arent possible!'
				return
		try:
			self["list"].changeDir(directory)
			if self.mode == 0:
				config.efb.lastDirB.value = directory
				self["description"].setText(_("Movies-> " + directory))
			elif self.mode == 1:
				config.efb.lastDirY.value = directory
				self["description"].setText(_("Files-> " + directory))
			elif self.mode == 2:
				config.efb.lastDirR.value = directory
				self["description"].setText(_("Pictures-> " + directory))
			elif self.mode == 3:
				config.efb.lastDirG.value = directory
				self["description"].setText(_("Music-> " + directory))
		except:
			pass
	
	def updateParent(self, directory):
		print 'updating Parent with directory %s ' % directory
		try:
			self["root_list"].changeDir(directory)
		except:
			pass

	def play(self):
		if self["list"].canDescent():
			return
		else:
			isPackage = False
			isScript = False
			if self.mode == 0:
				ev = self["list"].getServiceRef()
				self.session.nav.playService(ev)

			elif self.mode == 3 and config.efb.showMvi.getValue():
				ev = self["list"].getServiceRef()
				self.session.nav.playService(ev)
				self.getMVI()
				self.showBackgroundMVI()
				self.visibility()

			elif self.mode == 1:
				try:
					name = self["list"].getCurrentDirectory() + self["list"].getFilename()
					if len(name) > 7:
						if name[-7:] == '.tar.gz':
							isPackage = True
							self.session.open(Console, title="Extract Package", cmdlist = [ "tar -xzvf " + name + " -C /" ], closeOnSuccess = True)
							
					if len(name) > 8:
						if name[-8:] == '.tar.bz2':
							isPackage = True
							self.session.open(Console, title="Extract Package", cmdlist = [ "tar -xjvf " + name + " -C /" ], closeOnSuccess = True)
							
					if len(name) > 4:
						if name[-4:] == '.ipk':
							isPackage = True
							self.session.open(Console, title="Extract Package", cmdlist = [ "ipkg install " + name ], closeOnSuccess = True)
							
					if isPackage == False:
						if name.endswith(".sh"):
							self.choiceScript(name)
							return
						read = open(name, "r")
						for x in read:
							print x[:3]
							if x[:3] == "#!/":
								read.close()
								self.choiceScript(name)
								return
						read.close()
						
					if isPackage == False:
						self.session.openWithCallback(self.visibility, FileEdit, self["list"].getCurrentDirectory() + "/" + self["list"].getFilename())
					self.visibility()
				except:
					pass
			elif self.mode == 2:
				self.session.openWithCallback(self.visibility, PicView, self["list"].getFileList(), self["list"].getFilename(), self["list"].getCurrentDirectory())
				self.visibility()

	def choiceScript(self, script):
		tlist = []
		tlist.append((_("Edit Script"), "edit", script))
		tlist.append((_("Execute Script"), "exec_n", script))
		tlist.append((_("Execute Script with output"), "exec_o", script))
		tlist.append((_("Execute Script with output and Parameters"), "exec_obp", script))
		tlist.append((_("Execute Script in Background"), "exec_b", script))
		tlist.append((_("Execute Script in Background with output"), "exec_ob", script))
		self.session.openWithCallback(self.scriptChoice, ChoiceBox, title=_("You've selected a Script. Please make a decision:"), list = tlist)

	def scriptChoice(self, sel):
		if not sel is None:
			if sel[1] == 'edit':
				self.session.open(FileEdit, sel[2])
			elif sel[1] == 'exec_n':
				os.system(sel[2])
			elif sel[1] == 'exec_o':
				self.session.open(Console, title="Exec Script", cmdlist = [ sel[2] ])
			elif sel[1] == 'exec_b':
				order = sel[2] + ' &'
				os.system(order)
			elif sel[1] == 'exec_ob':
				self.session.open(Console, title="Exec Script in Background", cmdlist = [ sel[2] + ' &' ])
			elif sel[1] == 'exec_obp':
				self.session.openWithCallback(self.scriptParams, InputBox, title=_("Please enter the Parameters"), windowTitle =_("Exec Script"), text=sel[2], maxSize=False, visible_width = 56, type=Input.TEXT)
	
	def scriptParams(self, text):
		if not text is None:
			self.session.open(Console, title="Exec Script with Parameters", cmdlist = [ text ])

	def switchModeB(self):
		self["description"].setText(_("Active Mode: Movies"))
		self.switch(0)
	
	def switchModeY(self):
		self["description"].setText(_("Active Mode: Files"))
		self.switch(1)
	
	def switchModeR(self):
		self["description"].setText(_("Active Mode: Pictures"))
		self.switch(2)
		
	def switchModeG(self):
		self["description"].setText(_("Active Mode: Music"))
		self.switch(3)
	
	def switch(self, mod):
		path = None
		if mod == 0:
			self.mode = 0
			path = config.efb.lastDirB.value
			self["list"].useServiceRef = True
			self["list"].matchingPattern = self.filePattern
			config.efb.lastMode.value = 0
			self["m_back"].instance.setPixmapFromFile("/usr/lib/enigma2/python/Plugins/Extensions/MerlinBrowser/mode0.png")
			
		elif mod == 1:
			self.mode = 1
			path = config.efb.lastDirY.value
			self["list"].useServiceRef = False
			self["list"].matchingPattern = self.packagePattern
			config.efb.lastMode.value = 1
			self["m_back"].instance.setPixmapFromFile("/usr/lib/enigma2/python/Plugins/Extensions/MerlinBrowser/mode1.png")
			
		elif mod == 2:
			self.mode = 2
			path = config.efb.lastDirR.value
			self["list"].useServiceRef = False
			self["list"].matchingPattern = self.picturePattern
			config.efb.lastMode.value = 2
			self["m_back"].instance.setPixmapFromFile("/usr/lib/enigma2/python/Plugins/Extensions/MerlinBrowser/mode2.png")
		
		elif mod == 3:
			self.mode = 3
			path = config.efb.lastDirG.value
			self["list"].useServiceRef = True
			self["list"].matchingPattern = self.musicPattern
			config.efb.lastMode.value = 3
			self["m_back"].instance.setPixmapFromFile("/usr/lib/enigma2/python/Plugins/Extensions/MerlinBrowser/mode3.png")
		
		try:
			if not path is None:
				self.updateChild(path)
				self.updateParent(path)
			else:
				self["list"].changeDir("/")
		except:
			self["list"].changeDir("/")
				
	def visibility(self, force=1):
		if self.isVisible == True:
			self.isVisible = False
			self.hide()
		else:
			self.isVisible = True
			self.show()
			self["list"].refresh()

	def options(self):
		if self.isVisible == True:
			if self.currentList == "list":
				self.visibility()
				clid = []
				clid.append(self.clipData)
				clid.append(self.clipDataDir)
				self.session.openWithCallback(self.optionEnd, FileOptions,  self["list"].getFilename(), self["list"].getCurrentDirectory(), self["list"].canDescent(), clid)
			else:
				self.session.open(MB_Config)
	
	def optionEnd(self, copyList):
		print copyList
		self.clipData = copyList[0]
		self.clipDataDir = copyList[1]
		self.visibility()

	def find_last_of(self, name, sep):
		a=0
		b=0
		for x in name:
			a+=1
			if x == sep:
				b=a
		return b
			
	def Exit(self):
		if self.isVisible == False:
			self.show()
			self.isVisible = True
			return
		self.onExit = True
		config.efb.save()
		self.close()

def main(session, **kwargs):
	session.open(MerlinBrowser)

def menu(menuid, **kwargs):
	if menuid == "mainmenu":
		return [(_("Merlin Browser"), main, "merlin_browser", 45)]
	return []

def Plugins(**kwargs):
	return [
		PluginDescriptor(name = "Merlin Browser", description = "View, edit and execute files in your Dreambox", icon="filebrowser.png", where = PluginDescriptor.WHERE_PLUGINMENU, fnc = main),
		PluginDescriptor(name = "Merlin Browser", description = "View, edit and execute files in your Dreambox", icon="filebrowser.png", where = PluginDescriptor.WHERE_EXTENSIONSMENU, fnc=main)]
